#include "stdafx.h"
#include "resource.h"
#include "SampleApp.h"
#include "SettingsDlg.h"
#include <commctrl.h>

SampleApp	theApp;

//========================================================================
SampleApp::SampleApp()
{
	m_hDlg				= NULL;
	m_pDataManager		= NULL;
	m_pTwainInterface	= NULL;
	m_pSampleDlg		= NULL;
	m_pTransferManager	= NULL;
}

//========================================================================
SampleApp::~SampleApp()
{
	if(m_pTransferManager)
	{
		delete m_pTransferManager;
		m_pTransferManager = NULL;
	}

	if(m_pSampleDlg)
	{
		delete m_pSampleDlg;
		m_pSampleDlg = NULL;
	}

	if(m_pTwainInterface)
	{
		delete m_pTwainInterface;
		m_pTwainInterface = NULL;
	}

	if(m_pDataManager)
	{
		delete m_pDataManager;
		m_pDataManager = NULL;
	}

}

//========================================================================
BOOL SampleApp::Execute(HINSTANCE hInstance)
{
	INITCOMMONCONTROLSEX InitCtrls;
	InitCtrls.dwSize = sizeof(InitCtrls);
	InitCtrls.dwICC = ICC_WIN95_CLASSES | ICC_LISTVIEW_CLASSES;
	InitCommonControlsEx(&InitCtrls);

	if(StartUp(hInstance))
	{
		MainLoop();

		CloseDSM();
	}

	return TRUE;
}

//========================================================================
BOOL SampleApp::StartUp(HINSTANCE hInstance)
{
	BOOL bSuccess = TRUE;

	if(bSuccess)
	{
		if(!m_pDataManager)
		{
			m_pDataManager = new DataManager;
		}

		if(!m_pDataManager)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!m_pTwainInterface)
		{
			m_pTwainInterface = new TwainInterface;
		}

		if(!m_pTwainInterface)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!m_pSampleDlg)
		{
			m_pSampleDlg = new SampleDlg(hInstance, m_pDataManager);
		}

		if(!m_pSampleDlg)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		m_hDlg = NULL;
		m_hDlg = CreateDialogParam(hInstance, MAKEINTRESOURCE(IDD_MAIN_DLG), NULL, SampleDlgProc, (LPARAM)m_pSampleDlg);

		if(!m_hDlg)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!m_pTransferManager)
		{
			m_pTransferManager = new TransferManager(m_hDlg, m_pDataManager, m_pTwainInterface);
		}

		if(!m_pTransferManager)
		{
			bSuccess = FALSE;
		}
	}

	m_pSampleDlg->SetTransferManager(m_pTransferManager);

	if(bSuccess)
	{
		if(!m_pTwainInterface->connectDSM(m_hDlg))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		ShowWindow(m_hDlg, SW_SHOWNORMAL);
	}

	return bSuccess;
}

//========================================================================
int SampleApp::MainLoop(void)
{
	int		bRet	= 0;

	for(; ;)
	{
		Sleep(0);
		MSG Msg;

		if(1 != (bRet = GetMessage((LPMSG)&Msg, NULL, 0, 0)))
		{
			break;	// WM_QUIT or error;
		}

		if(!EventProc(&Msg))
		{
			if(!IsDialogMessage(m_hDlg, (LPMSG)&Msg))
			{
				TranslateMessage((LPMSG)&Msg);
				DispatchMessage((LPMSG)&Msg);
			}
			else
			{
				switch(Msg.message)
				{
					case WM_ONACQUIRE:
					{
						Acquire();
						break;
					}
					case WM_SELECTSOURCE:
					{
						TW_STR32	dsProductName;
						SelectSource(dsProductName);
						m_pSampleDlg->UpdateSourceTxt((LPCTSTR)dsProductName);
						ChangeState();
						break;
					}
					case WM_ONSETTINGS:
					{
						SettingsDlg *pSettingsDlg = NULL;
						HINSTANCE	hInstance;
						hInstance = (HINSTANCE)GetWindowLong(m_pSampleDlg->GetHWND(), GWL_HINSTANCE);
						pSettingsDlg = new SettingsDlg(m_pTwainInterface);

						DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_SETTINGS_DLG), m_pSampleDlg->GetHWND(), SettingsDlgProc, (LPARAM)pSettingsDlg);

						delete pSettingsDlg;
						pSettingsDlg = NULL;
						break;
					}

				}
			}
		}
	}

	return 0;
}

//========================================================================
BOOL SampleApp::Acquire(void)
{
	BOOL	bSuccess	= TRUE;

	if(bSuccess)
	{
		if(!m_pTwainInterface->Acquire(m_hDlg, m_pSampleDlg->GetbShowUI()))
		{
			bSuccess = FALSE;
			ChangeState();
		}
	}

	return bSuccess;
}

//========================================================================
BOOL SampleApp::DisableDS(void)
{
	BOOL	bSuccess	= TRUE;

	if(bSuccess)
	{
		if(!m_pTwainInterface->disableDS(m_hDlg))
		{
			bSuccess = FALSE;
		}
	}

	return bSuccess;
}

//========================================================================
BOOL SampleApp::CloseDS(void)
{
	BOOL	bSuccess	= TRUE;

	if(bSuccess)
	{
		if(!m_pTwainInterface->unloadDS(m_hDlg))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
		ChangeState();

	return bSuccess;
}

//========================================================================
BOOL SampleApp::CloseDSM(void)
{
	BOOL	bSuccess	= TRUE;

	if(bSuccess)
	{
		if(!m_pTwainInterface->disconnectDSM(m_hDlg))
		{
			bSuccess = FALSE;
		}
	}

	return bSuccess;
}

//========================================================================
BOOL SampleApp::EventProc(MSG *lpMsg)
{
	TW_UINT16	twrc			= TWRC_NOTDSEVENT;
	TW_EVENT	twEvent;
	TW_INT16	MessageSelect	= 0;

	memset(&twEvent, 0, sizeof(TW_EVENT));

	if(m_pTwainInterface->IsEnabledDS())
	{
		twEvent.pEvent = (TW_MEMREF)lpMsg;

		twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
											DAT_EVENT,
											MSG_PROCESSEVENT,
											&twEvent);

		MessageSelect = twEvent.TWMessage;

		switch(MessageSelect)
		{
			case MSG_XFERREADY:
				m_pTransferManager->NativeTransfer();
				break;

			case MSG_CLOSEDSREQ:
			case MSG_CLOSEDSOK:
				DisableDS();
				break;

			case MSG_NULL:
			default:
				break;
		}
	}

	return (twrc == TWRC_DSEVENT);           // returns TRUE or FALSE
}

//========================================================================
BOOL SampleExecute(HINSTANCE hInstance)
{
	return theApp.Execute(hInstance);
}

//========================================================================
BOOL SampleApp::SelectSource(pTW_STR32 pSourceName)
{
	BOOL		bSuccess	= TRUE;
	TW_IDENTITY	twDataSource;

	memset(&twDataSource, 0, sizeof(twDataSource));

	if(bSuccess)
	{
		if(!m_pTwainInterface->selectDS(m_hDlg,&twDataSource))
		{
			bSuccess = FALSE;
			ChangeState();
		}
	}

	if(bSuccess)
	{
		memcpy_s(pSourceName,sizeof(TW_STR32),twDataSource.ProductName,sizeof(TW_STR32));
	}
	else
	{
		pSourceName[0] = NULL;
	}

	return bSuccess;
}

//========================================================================
void SampleApp::ChangeState(void)
{
	m_pSampleDlg->ChangeState(m_pTwainInterface->GetStateCode());
}