#include "stdafx.h"
#include "resource.h"
#include "SampleApp.h"
#include "SampleDlg.h"
#include <commctrl.h>

#define IMAGE_CTRL_NUM								2
#define IMAGE_CTRL_FRONT							0
#define IMAGE_CTRL_REAR								1
#define LIST_COLUMN_NUM								3
#define LIST_COLUMN_WIDTH_TYPE						100
#define LIST_COLUMN_WIDTH_DATA						150
#define LIST_COLUMN_WIDTH_COORDINATE				150

#define IDS_LIST_COLUMN_TYPE						_T("Type")
#define IDS_LIST_COLUMN_DATA						_T("Data")
#define IDS_LIST_COLUMN_COORDINATE					_T("Coordinate (X, Y)")
#define IDS_RECOG_NODATA							_T("No Data")
#define IDS_RECOG_COORDINATE_FORMAT					_T("%d, %d")
#define	IDS_RECOG_BARCODETYPE_3OF9					_T("Code39")
#define	IDS_RECOG_BARCODETYPE_2OF5INTERLEAVED		_T("ITF")
#define	IDS_RECOG_BARCODETYPE_2OF5NONINTERLEAVED	_T("2of5NonInterLeaved")
#define	IDS_RECOG_BARCODETYPE_CODE93				_T("Code93")
#define	IDS_RECOG_BARCODETYPE_CODE128				_T("Code128")
#define	IDS_RECOG_BARCODETYPE_UCC128				_T("Ucc128")
#define	IDS_RECOG_BARCODETYPE_CODABAR				_T("Codabar")
#define	IDS_RECOG_BARCODETYPE_UPCA					_T("UPC-A")
#define	IDS_RECOG_BARCODETYPE_UPCE					_T("UPC-E")
#define	IDS_RECOG_BARCODETYPE_EAN8					_T("EAN8")
#define	IDS_RECOG_BARCODETYPE_EAN13					_T("EAN13")
#define	IDS_RECOG_BARCODETYPE_POSTNET				_T("Postnet")
#define	IDS_RECOG_BARCODETYPE_PDF417				_T("PDF417")
#define	IDS_RECOG_BARCODETYPE_2OF5INDUSTRIAL		_T("2of5Industrial")
#define	IDS_RECOG_BARCODETYPE_2OF5MATRIX			_T("2of5Matrix")
#define	IDS_RECOG_BARCODETYPE_2OF5DATALOGIC			_T("2of5DataLogic")
#define	IDS_RECOG_BARCODETYPE_2OF5IATA				_T("2of5Iata")
#define	IDS_RECOG_BARCODETYPE_3OF9FULLASCII			_T("3of9FullAscii")
#define	IDS_RECOG_BARCODETYPE_CODABARWITHSTARTSTOP	_T("CodabarWithStartStop")
#define	IDS_RECOG_BARCODETYPE_MAXICODE				_T("Maxicode")
#define	IDS_RECOG_BARCODETYPE_UNKNOWN				_T("Unknown Type")

#define STR_ERROR_TITLE			_T("Error")
#define STR_ERROR_SELECTSOURCE	_T("Fail to load DS")

//========================================================================
SampleDlg::SampleDlg(HINSTANCE hInstance, DataManager *pDataManager)
{
	m_hDlg				= NULL;
	m_hInstance			= hInstance;
	m_pDataManager		= pDataManager;
	m_nImageNum			= 0;
	m_nImageWidthFront	= 0;
	m_nImageHeightFront	= 0;
	m_nImageWidthRear	= 0;
	m_nImageHeightRear	= 0;
	m_hDCFront			= NULL;
	m_hDCRear			= NULL;
	m_hbmFront			= NULL;
	m_hbmRear			= NULL;
	m_pTransferManager	= NULL;
	m_bShowUI			= TRUE;
}

//========================================================================
SampleDlg::~SampleDlg(void)
{
	if(m_hDCRear)
	{
		DeleteDC(m_hDCRear);
		m_hDCRear = NULL;
	}

	if(m_hDCFront)
	{
		DeleteDC(m_hDCFront);
		m_hDCFront = NULL;
	}

	if(m_hbmRear)
	{
		DeleteObject(m_hbmRear);
		m_hbmRear = NULL;
	}

	if(m_hbmFront)
	{
		DeleteObject(m_hbmFront);
		m_hbmFront = NULL;
	}
}

//========================================================================
BOOL SampleDlg::OnInitDialog(HWND hDlg)
{
	BOOL			bSuccess		= TRUE;

	if(bSuccess)
	{
		if(!hDlg)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		m_hDlg = hDlg;

		if(!InitView(TRUE))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!InitRecog(TRUE))
		{
			bSuccess = FALSE;
		}
	}

	{
		HICON	hIcon	= NULL;

		if(bSuccess)
		{
			hIcon = (HICON)LoadImage(m_hInstance, MAKEINTRESOURCE(IDI_SAMPLEPROGRAM), IMAGE_ICON, 0, 0, LR_DEFAULTSIZE);
			if(!hIcon)
			{
				bSuccess = FALSE;
			}
		}

		if(bSuccess)
		{
			SendMessage(hDlg, WM_SETICON, ICON_BIG, (LPARAM)hIcon);
		}
	}

	if(bSuccess)
	{
		CheckDlgButton(hDlg, IDC_SHOWUI_CHECK, BST_CHECKED);
		EnableWindow(GetDlgItem(hDlg, IDC_SHOWUI_CHECK), FALSE);
		EnableWindow(GetDlgItem(hDlg, IDC_SETTINGS_BTN), FALSE);
		EnableWindow(GetDlgItem(hDlg, IDC_ACQUIRE_BTN), FALSE);
	}

	return bSuccess;
}

//========================================================================
void SampleDlg::OnBtnAcquire(void)
{
	PostMessage(m_hDlg, (UINT)WM_ONACQUIRE, NULL, NULL);
}

//========================================================================
void SampleDlg::ChangeState(STATECODE state)
{
	BOOL		bEnable	= TRUE;

	if(state >= STATE_LOAD_DS)
		bEnable = TRUE;
	else
		bEnable = FALSE;

	EnableWindow(GetDlgItem(m_hDlg, IDC_ACQUIRE_BTN), bEnable);
	EnableWindow(GetDlgItem(m_hDlg, IDC_SHOWUI_CHECK),bEnable);
	SettingsButtonUpdate();

	EnableMenuItem(GetSystemMenu(m_hDlg, FALSE), SC_CLOSE, MF_ENABLED);

}

//========================================================================
void SampleDlg::ScanFinish(void)
{
	m_nImageNum = 0;
}

//========================================================================
void SampleDlg::Update()
{
	BOOL		bSuccess	= TRUE;

	if(0 == m_nImageNum)
	{
		if(bSuccess)
		{
			if(!InitView())
			{
				bSuccess = FALSE;
			}
		}

		if(bSuccess)
		{
			if(!InitRecog())
			{
				bSuccess = FALSE;
			}
		}
	}
	else
	{
		if(bSuccess)
		{
			if(!InitViewSub(m_nImageNum % IMAGE_CTRL_NUM))
			{
				bSuccess = FALSE;
			}
		}
	}

	if(bSuccess)
	{
		if(!UpdateView())
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!UpdateMicrData())
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!UpdateOcrData())
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!UpdateBarcodeData())
		{
			bSuccess = FALSE;
		}
	}

	m_nImageNum++;
}

//========================================================================
void SampleDlg::OnBtnUIModeChange(void)
{
	if(IsDlgButtonChecked(m_hDlg, IDC_SHOWUI_CHECK))
	{
		m_bShowUI = TRUE;
	}
	else
	{
		m_bShowUI = FALSE;
	}

	SettingsButtonUpdate();

}

//========================================================================
void SampleDlg::SettingsButtonUpdate(void)
{
	if(m_bShowUI)
	{
		EnableWindow(GetDlgItem(m_hDlg, IDC_SETTINGS_BTN), FALSE);
	}
	else
	{
		EnableWindow(GetDlgItem(m_hDlg, IDC_SETTINGS_BTN), TRUE);
	}
}

//========================================================================
void SampleDlg::OnBtnSettings(HWND hDlg)
{
	PostMessage(hDlg, (UINT)WM_ONSETTINGS, NULL, NULL);

}

//========================================================================
BOOL SampleDlg::UpdateMicrData(void)
{
	BOOL			bSuccess	= TRUE;
	EPMICR_DATA		micr;
	HWND			hWnd		= NULL;
	TW_STR255		str			= _T("");
	RECOG_STATE		state		= RECOG_DISABLE;
	BOOL			bShow		= FALSE;
	memset(&micr, 0x00, sizeof(micr));

	if(bSuccess)
	{
		hWnd = GetDlgItem(m_hDlg, (0 == m_nImageNum % IMAGE_CTRL_NUM) ? IDC_FRONT_MICR_EDIT : IDC_REAR_MICR_EDIT);
		if(!hWnd)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!SetWindowText(hWnd, str))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		state = m_pDataManager->GetDecodeMicr(&micr);
		switch(state)
		{
			default:
			case RECOG_DISABLE:
				bShow = FALSE;
				break;

			case RECOG_ENABLE:
				bSuccess = !(strcpy_s(str, sizeof(TW_STR255), IDS_RECOG_NODATA));
				bShow = TRUE;
				break;

			case RECOG_DECORD:
				bSuccess = !(strcpy_s(str, sizeof(TW_STR255), micr.magData));
				bShow = TRUE;
				break;
		}
	}

	if(bSuccess && bShow)
	{
		if(!SetWindowText(hWnd, str))
		{
			bSuccess = FALSE;
		}
	}

	return bSuccess;
}

//========================================================================
BOOL SampleDlg::UpdateOcrData(void)
{
	BOOL			bSuccess	= TRUE;
	EPOCR_DATA		ocr;
	HWND			hWnd		= NULL;
	TW_STR255		str			= _T("");
	RECOG_STATE		state		= RECOG_DISABLE;
	BOOL			bShow		= FALSE;
	memset(&ocr, 0x00, sizeof(ocr));

	if(bSuccess)
	{
		hWnd = GetDlgItem(m_hDlg, (0 == m_nImageNum % IMAGE_CTRL_NUM) ? IDC_FRONT_OCR_EDIT : IDC_REAR_OCR_EDIT);
		if(!hWnd)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!SetWindowText(hWnd, str))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		state = m_pDataManager->GetDecodeOcr(&ocr);
		switch(state)
		{
			default:
			case RECOG_DISABLE:
				bShow = FALSE;
				break;

			case RECOG_ENABLE:
				bSuccess = !(strcpy_s(str, sizeof(TW_STR255), IDS_RECOG_NODATA));
				bShow = TRUE;
				break;

			case RECOG_DECORD:
				bSuccess = !(strcpy_s(str, sizeof(TW_STR255), ocr.Data));
				bShow = TRUE;
				break;
		}
	}

	if(bSuccess && bShow)
	{
		if(!SetWindowText(hWnd, str))
		{
			bSuccess = FALSE;
		}
	}

	return bSuccess;
}

//========================================================================
BOOL SampleDlg::UpdateBarcodeData(void)
{
	BOOL				bSuccess	= TRUE;
	TW_UINT32			targetID	= 0;
	EPBarcodeDataList	barcode;
	HWND				hWnd		= NULL;
	TW_STR255			str			= _T("");
	RECOG_STATE			state		= RECOG_DISABLE;
	LVITEM				item;
	UINT				i			= 0;

	item.mask = LVIF_TEXT;

	if(bSuccess)
	{
		switch(m_nImageNum % IMAGE_CTRL_NUM)
		{
			case IMAGE_CTRL_FRONT:
				targetID = IDC_FRONT_BARCODE_LIST;
				break;

			case IMAGE_CTRL_REAR:
				targetID = IDC_REAR_BARCODE_LIST;
				break;

			default:
				bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		hWnd = GetDlgItem(m_hDlg, targetID);
		if(!hWnd)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!ListView_DeleteAllItems(hWnd))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		state = m_pDataManager->GetDecodeBarcode(&barcode);
		switch(state)
		{
			default:
			case RECOG_DISABLE:
				break;

			case RECOG_ENABLE:
				bSuccess = !(strcpy_s(str, sizeof(TW_STR255), IDS_RECOG_NODATA));

				if(bSuccess)
				{
					item.pszText	= _T("");
					item.iItem		= 0;
					item.iSubItem	= 0;
					ListView_InsertItem(hWnd, &item);

					item.pszText	= str;
					item.iItem		= 0;
					item.iSubItem	= 1;
					ListView_SetItem(hWnd, &item);

					item.pszText	= _T("");
					item.iItem		= 0;
					item.iSubItem	= 2;
					ListView_SetItem(hWnd, &item);
				}
				break;

			case RECOG_DECORD:
				for(i = 0; i < barcode.size(); i++)
				{
					if(bSuccess)
					{
						if(!GetBarcodeTypeString(barcode[i].Type, &str))
						{
							bSuccess = FALSE;
						}
					}

					if(bSuccess)
					{
						item.pszText	= str;
						item.iItem		= i;
						item.iSubItem	= 0;
						ListView_InsertItem(hWnd, &item);

						item.pszText	= barcode[i].Text;
						item.iItem		= i;
						item.iSubItem	= 1;
						ListView_SetItem(hWnd, &item);

						if(-1 == sprintf_s(str, IDS_RECOG_COORDINATE_FORMAT, barcode[i].BarcodeX, barcode[i].BarcodeY))
						{
							bSuccess = FALSE;
						}
					}

					if(bSuccess)
					{
						item.pszText	= str;
						item.iItem		= i;
						item.iSubItem	= 2;
						ListView_SetItem(hWnd, &item);
					}
				}
				break;
		}
	}

	return bSuccess;
}

//========================================================================
BOOL SampleDlg::GetBarcodeTypeString(TW_UINT32 Type, TW_STR255 *pStr)
{
	BOOL				bSuccess	= TRUE;

	if(bSuccess)
	{
		switch(Type)
		{
			case TWBT_3OF9:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_3OF9));
				break;

			case TWBT_2OF5INTERLEAVED:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_2OF5INTERLEAVED));
				break;

			case TWBT_2OF5NONINTERLEAVED:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_2OF5NONINTERLEAVED));
				break;

			case TWBT_CODE93:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_CODE93));
				break;

			case TWBT_CODE128:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_CODE128));
				break;

			case TWBT_UCC128:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_UCC128));
				break;

			case TWBT_CODABAR:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_CODABAR));
				break;

			case TWBT_UPCA:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_UPCA));
				break;

			case TWBT_UPCE:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_UPCE));
				break;

			case TWBT_EAN8:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_EAN8));
				break;

			case TWBT_EAN13:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_EAN13));
				break;

			case TWBT_POSTNET:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_POSTNET));
				break;

			case TWBT_PDF417:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_PDF417));
				break;

			case TWBT_2OF5INDUSTRIAL:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_2OF5INDUSTRIAL));
				break;

			case TWBT_2OF5MATRIX:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_2OF5MATRIX));
				break;

			case TWBT_2OF5DATALOGIC:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_2OF5DATALOGIC));
				break;

			case TWBT_2OF5IATA:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_2OF5IATA));
				break;

			case TWBT_3OF9FULLASCII:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_3OF9FULLASCII));
				break;

			case TWBT_CODABARWITHSTARTSTOP:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_CODABARWITHSTARTSTOP));
				break;

			case TWBT_MAXICODE:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_MAXICODE));
				break;

			default:
				bSuccess = !(strcpy_s(*pStr, sizeof(TW_STR255), IDS_RECOG_BARCODETYPE_UNKNOWN));
				break;
		}
	}

	return bSuccess;
}

//========================================================================
BOOL SampleDlg::UpdateView(void)
{
	BOOL			bSuccess		= TRUE;
	HBITMAP			hBitMapOrg		= NULL;
	TW_UINT32		targetID		= 0;
	HWND			hWnd			= NULL;
	LONG			nImageWidth		= 0;
	LONG			nImageHeight	= 0;
	LONG			nRectWidth		= 0;
	LONG			nRectHeight		= 0;
	HBITMAP			hBitMapFix		= NULL;
	double			ratio			= 0.0;
	HDC             hDC				= NULL,
							 hMemDC			= NULL,
									hOffScreen		= NULL;
	RECT			Rect;
	RECT			dstRect;
	memset(&Rect, 0x00, sizeof(Rect));
	memset(&dstRect, 0x00, sizeof(dstRect));

	hBitMapOrg = m_pDataManager->GetBitmapHandle();

	if(bSuccess)
	{
		if(!hBitMapOrg)
		{
			InvalidateRect(m_hDlg, NULL, FALSE);
			UpdateWindow(m_hDlg);
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		switch(m_nImageNum % IMAGE_CTRL_NUM)
		{
			case IMAGE_CTRL_FRONT:
				targetID = IDC_FRONT_PIC;
				break;

			case IMAGE_CTRL_REAR:
				targetID = IDC_REAR_PIC;
				break;

			default:
				bSuccess = FALSE;
				break;
		}
	}

	if(bSuccess)
	{
		hWnd = GetDlgItem(m_hDlg, targetID);
		if(!hWnd)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		hDC = GetDC(hWnd);
		if(!hDC)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		hBitMapFix = FixUp(hWnd, hBitMapOrg);
		if(!hBitMapFix)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		switch(m_nImageNum % IMAGE_CTRL_NUM)
		{
			case IMAGE_CTRL_FRONT:
				nImageWidth		= m_nImageWidthFront;
				nImageHeight	= m_nImageHeightFront;
				hOffScreen		= m_hDCFront;
				break;

			case IMAGE_CTRL_REAR:
				nImageWidth		= m_nImageWidthRear;
				nImageHeight	= m_nImageHeightRear;
				hOffScreen		= m_hDCRear;
				break;

			default:
				bSuccess = FALSE;
				break;
		}

		if(!hOffScreen)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!GetClientRect(hWnd, &Rect))
		{
			bSuccess = FALSE;
		}
	}

	{
		HBRUSH  hSolidBrush = NULL;

		if(bSuccess)
		{
			hSolidBrush = CreateSolidBrush(RGB(0x80, 0x80, 0x80));
			if(!hSolidBrush)
			{
				bSuccess = FALSE;
			}
		}

		if(bSuccess)
		{
			if(!FillRect(hOffScreen, &Rect, hSolidBrush))
			{
				bSuccess = FALSE;
			}
			DeleteObject(hSolidBrush);
			hSolidBrush = NULL;
		}
	}

	if(bSuccess)
	{
		nRectWidth	= (Rect.right-Rect.left);
		nRectHeight = (Rect.bottom-Rect.top);

		if(0 > nRectWidth)
		{
			nRectWidth = 1;
		}

		if(0 > nRectHeight)
		{
			nRectHeight = 1;
		}

		ratio = max((double)nImageWidth / (double)nRectWidth,
					(double)nImageHeight / (double)nRectHeight);

		dstRect.left	= 0;
		dstRect.top		= 0;

		if(1.0 < ratio)
		{
			dstRect.right	= (LONG)(nImageWidth / ratio);
			dstRect.bottom	= (LONG)(nImageHeight / ratio);
		}
		else
		{
			dstRect.right	= nImageWidth;
			dstRect.bottom	= nImageHeight;
		}

		hMemDC = CreateCompatibleDC(hDC);
		if(!hMemDC)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!(SelectObject(hMemDC, hBitMapFix)))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!(SetStretchBltMode(hOffScreen, HALFTONE)))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!(StretchBlt(hOffScreen,
						  0,
						  0,
						  (dstRect.right-dstRect.left),
						  (dstRect.bottom-dstRect.top),
						  hMemDC,
						  0,
						  0,
						  nImageWidth,
						  nImageHeight,
						  SRCCOPY)))
		{
			bSuccess = FALSE;
		}
	}

	if(hMemDC)
	{
		DeleteDC(hMemDC);
		hMemDC = NULL;
	}

	if(hBitMapFix)
	{
		DeleteObject(hBitMapFix);
		hBitMapFix = NULL;
	}

	if(hDC)
	{
		ReleaseDC(hWnd, hDC);
		hDC = NULL;
	}

	if(bSuccess)
	{
		if(!InvalidateRect(m_hDlg, NULL, FALSE))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(!UpdateWindow(m_hDlg))
		{
			bSuccess = FALSE;
		}
	}

	m_pDataManager->FreeBitmapHandle();

	return bSuccess;
}

//========================================================================
BOOL SampleDlg::InitView(BOOL bFirst/*=FALSE*/)
{
	BOOL			bSuccess	= TRUE;

	if(bFirst)
	{
		if(m_hDCFront)
		{
			DeleteDC(m_hDCFront);
			m_hDCFront = NULL;
		}

		if(m_hDCRear)
		{
			DeleteDC(m_hDCRear);
			m_hDCRear = NULL;
		}

		if(m_hbmFront)
		{
			DeleteObject(m_hbmFront);
			m_hbmFront = NULL;
		}

		if(m_hbmRear)
		{
			DeleteObject(m_hbmRear);
			m_hbmRear = NULL;
		}
	}

	for(int i = 0; i < IMAGE_CTRL_NUM; i++)
	{
		if(bSuccess)
		{
			if(!InitViewSub(i, bFirst))
			{
				bSuccess = FALSE;
			}
		}
	}

	if(bSuccess)
	{
		bSuccess = InvalidateRect(m_hDlg, NULL, FALSE);
	}

	return bSuccess;
}

//========================================================================
BOOL SampleDlg::InitViewSub(TW_UINT32 targetView, BOOL bFirst/*=FALSE*/)
{
	BOOL			bSuccess	= TRUE;
	HWND			hWnd		= NULL;
	HDC				hDC			= NULL;
	HDC				hOffScreen	= NULL;
	RECT			Rect;

	memset(&Rect, 0, sizeof(Rect));

	if(bSuccess)
	{
		TW_UINT32	targetViewID	= 0;

		switch(targetView)
		{
			case IMAGE_CTRL_FRONT:
				targetViewID = IDC_FRONT_PIC;
				break;

			case IMAGE_CTRL_REAR:
				targetViewID = IDC_REAR_PIC;
				break;

			default:
				break;
		}

		hWnd = GetDlgItem(m_hDlg, targetViewID);
		if(!hWnd)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		hDC = GetDC(hWnd);
		if(!hDC)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		bSuccess = GetClientRect(hWnd, &Rect);
	}

	if(bFirst)
	{
		HDC			hDCTmp		= NULL;
		HBITMAP		hBitmapTmp	= NULL;

		if(bSuccess)
		{
			hDCTmp = CreateCompatibleDC(hDC);
			if(!hDCTmp)
			{
				bSuccess = FALSE;
			}
		}

		if(bSuccess)
		{
			hOffScreen = hDCTmp;
		}

		if(bSuccess)
		{
			hBitmapTmp = CreateCompatibleBitmap(hDC, (Rect.right-Rect.left), (Rect.bottom-Rect.top));
			if(!hBitmapTmp)
			{
				bSuccess = FALSE;
			}
		}

		if(bSuccess)
		{
			switch(targetView)
			{
				case IMAGE_CTRL_FRONT:
					m_hDCFront = hDCTmp;
					m_hbmFront = hBitmapTmp;
					break;

				case IMAGE_CTRL_REAR:
					m_hDCRear = hDCTmp;
					m_hbmRear = hBitmapTmp;
					break;

				default:
					break;
			}

			if(!(SelectObject(hOffScreen, hBitmapTmp)))
			{
				bSuccess = FALSE;
			}
		}
	}
	else
	{
		if(bSuccess)
		{
			switch(targetView)
			{
				case IMAGE_CTRL_FRONT:
					hOffScreen = m_hDCFront;
					break;

				case IMAGE_CTRL_REAR:
					hOffScreen = m_hDCRear;
					break;

				default:
					break;
			}

			if(!hOffScreen)
			{
				bSuccess = FALSE;
			}
		}
	}

	{
		HBRUSH hSolidBrush = NULL;

		if(bSuccess)
		{
			hSolidBrush = CreateSolidBrush(RGB(0x80, 0x80, 0x80));
			if(!hSolidBrush)
			{
				bSuccess = FALSE;
			}
		}

		if(bSuccess)
		{
			if(!FillRect(hOffScreen, &Rect, hSolidBrush))
			{
				bSuccess = FALSE;
			}
			DeleteObject(hSolidBrush);
			hSolidBrush = NULL;
		}
	}

	if(hDC)
	{
		ReleaseDC(hWnd, hDC);
		hDC = NULL;
	}

	return bSuccess;
}

//========================================================================
BOOL SampleDlg::InitRecog(BOOL bFirst/*=FALSE*/)
{
	BOOL			bSuccess		= TRUE;
	LVCOLUMN		lvcolumn;
	HWND			hList			= NULL;
	TW_UINT32		nColumn			= 0;
	TW_UINT32		targetMicrID	= 0;
	TW_UINT32		targetOcrID		= 0;
	TW_UINT32		targetBarcodeID	= 0;
	int				columWidth[]	= { LIST_COLUMN_WIDTH_TYPE,
										LIST_COLUMN_WIDTH_DATA,
										LIST_COLUMN_WIDTH_COORDINATE
						  };
	LPTSTR			strTitle[]		= { IDS_LIST_COLUMN_TYPE,
										IDS_LIST_COLUMN_DATA,
										IDS_LIST_COLUMN_COORDINATE
						   };

	for(int i = 0; i < IMAGE_CTRL_NUM; i++)
	{
		if(bSuccess)
		{
			switch(i)
			{
				case IMAGE_CTRL_FRONT:
					targetMicrID	= IDC_FRONT_MICR_EDIT;
					targetOcrID		= IDC_FRONT_OCR_EDIT;
					targetBarcodeID	= IDC_FRONT_BARCODE_LIST;
					break;

				case IMAGE_CTRL_REAR:
					targetMicrID	= IDC_REAR_MICR_EDIT;
					targetOcrID		= IDC_REAR_OCR_EDIT;
					targetBarcodeID	= IDC_REAR_BARCODE_LIST;
					break;

				default:
					bSuccess = FALSE;
					break;
			}
		}

		if(bSuccess)
		{
			if(bFirst)
			{
				hList = GetDlgItem(m_hDlg, targetBarcodeID);

				lvcolumn.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
				lvcolumn.fmt  = LVCFMT_LEFT;

				for(nColumn = 0; nColumn < LIST_COLUMN_NUM; nColumn++)
				{
					lvcolumn.cx			= columWidth[nColumn];
					lvcolumn.pszText	= strTitle[nColumn];
					lvcolumn.iSubItem	= nColumn;
					ListView_InsertColumn(hList, nColumn, &lvcolumn);
				}
			}

			if((!SetWindowText(GetDlgItem(m_hDlg, targetMicrID), _T("")))	||
					(!SetWindowText(GetDlgItem(m_hDlg, targetOcrID), _T("")))	||
					(!ListView_DeleteAllItems(GetDlgItem(m_hDlg, targetBarcodeID))))
			{
				bSuccess = FALSE;
			}
		}
	}

	return bSuccess;
}

//========================================================================
void SampleDlg::OnPaint(void)
{
	BOOL			bSuccess	= TRUE;
	HWND			hWnd		= NULL;
	RECT			Rect;
	PAINTSTRUCT     ps;
	HDC             hDC			= NULL;
	HDC				hOffScreen	= NULL;
	TW_UINT32		targetID	= 0;
	memset(&Rect, 0x00, sizeof(Rect));
	memset(&ps, 0x00, sizeof(ps));

	for(int i = 0; i < IMAGE_CTRL_NUM; i++)
	{
		hWnd		= NULL;
		memset(&Rect, 0x00, sizeof(Rect));
		hDC			= NULL;
		hOffScreen	= NULL;

		if(bSuccess)
		{
			switch(i)
			{
				case IMAGE_CTRL_FRONT:
					targetID	= IDC_FRONT_PIC;
					hOffScreen	= m_hDCFront;
					break;

				case IMAGE_CTRL_REAR:
					targetID	= IDC_REAR_PIC;
					hOffScreen	= m_hDCRear;
					break;

				default:
					bSuccess = FALSE;
					break;
			}
		}

		if(bSuccess)
		{
			hWnd = GetDlgItem(m_hDlg, targetID);

			if(!hWnd)
			{
				bSuccess = FALSE;
			}
		}

		if(bSuccess)
		{
			if(!hOffScreen)
			{
				bSuccess = FALSE;
			}
		}

		if(bSuccess)
		{
			if(!GetClientRect(hWnd, &Rect))
			{
				bSuccess = FALSE;
			}
		}

		if(bSuccess)
		{
			hDC = BeginPaint(hWnd, &ps);

			BitBlt(hDC,
					 0,
					 0,
					 (Rect.right-Rect.left),
					 (Rect.bottom-Rect.top),
					 hOffScreen,
					 0,
					 0,
					 SRCCOPY);

			EndPaint(hWnd, &ps);
		}
	}
}

//========================================================================
HBITMAP SampleDlg::FixUp(HWND hWnd, HANDLE hDib)
{
	BOOL				bSuccess			= TRUE;
	LPBITMAPINFOHEADER  lpDib				= NULL,
								   lpBi				= NULL;
	LPSTR               lpBits				= NULL;
	DWORD               dwColorTableSize	= 0;
	HBITMAP	            hBitMap				= NULL;
	HDC                 hDC					= NULL;

	if(bSuccess)
	{
		if(m_nImageNum % IMAGE_CTRL_NUM == IMAGE_CTRL_FRONT)
		{
			m_nImageWidthFront	= 0;
			m_nImageHeightFront = 0;
		}
		else
		{
			m_nImageWidthRear	= 0;
			m_nImageHeightRear	= 0;
		}
	}

	if(bSuccess)
	{
		if((!hWnd) ||
				(!hDib))
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		lpDib = (LPBITMAPINFOHEADER)GlobalLock(hDib);
		if(!lpDib)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		lpBi = lpDib;

		dwColorTableSize = (DWORD)(DibNumColors(lpDib) * sizeof(RGBQUAD));
		lpBits = (LPSTR)lpDib + lpBi->biSize + dwColorTableSize;

		if(m_nImageNum % IMAGE_CTRL_NUM == IMAGE_CTRL_FRONT)
		{
			m_nImageWidthFront	= lpBi->biWidth;
			m_nImageHeightFront = lpBi->biHeight;
		}
		else
		{
			m_nImageWidthRear	= lpBi->biWidth;
			m_nImageHeightRear	= lpBi->biHeight;
		}

		hDC = GetDC(hWnd);
		if(!hDC)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		if(1 == lpDib->biBitCount)
		{
			hBitMap = CreateBitmap((int)lpDib->biWidth, (int)lpDib->biHeight, 1, 1, lpBits);
			if(hBitMap)
			{
				SetDIBits(hDC, hBitMap, 0, (UINT)lpDib->biHeight, lpBits,(LPBITMAPINFO)lpDib, DIB_RGB_COLORS);
			}
		}
		else
		{
			hBitMap = CreateDIBitmap(hDC,
									   lpDib,
									   (LONG)CBM_INIT,
									   (LPSTR)lpBits,
									   (LPBITMAPINFO)lpDib,
									   DIB_RGB_COLORS);
		}

		GlobalUnlock(hDib);
		ReleaseDC(hWnd, hDC);
		hDC = NULL;
	}

	if(!bSuccess)
	{
		hBitMap = NULL;
	}

	return (hBitMap);
}

//========================================================================
WORD SampleDlg::DibNumColors(VOID FAR *pv)
{
	BOOL				bSuccess	= TRUE;
	int					Bits		= 0;
	WORD				wRet		= 0;
	LPBITMAPINFOHEADER	lpbi		= NULL;
	LPBITMAPCOREHEADER	lpbc		= NULL;

	if(!pv)
	{
		wRet = (WORD)-1;
		bSuccess = FALSE;
	}

	if(bSuccess)
	{
		lpbi = ((LPBITMAPINFOHEADER)pv);
		lpbc = ((LPBITMAPCOREHEADER)pv);

		if(lpbi->biSize != sizeof(BITMAPCOREHEADER))
		{
			if(lpbi->biClrUsed != 0)
			{
				wRet = (WORD)lpbi->biClrUsed;
				bSuccess = FALSE;
			}
			else
			{
				Bits = lpbi->biBitCount;
			}
		}
		else
		{
			Bits = lpbc->bcBitCount;
		}
	}

	if(bSuccess)
	{
		switch(Bits)
		{
			case 1:
				wRet = 2;
				break;

			case 4:
				wRet = 16;
				break;

			case 8:
				wRet = 256;
				break;

			default:
				wRet = 0;
				break;
		}
	}

	return wRet;
}

//========================================================================
INT_PTR CALLBACK SampleDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	SampleDlg	*pSampleDlg	= NULL;
	INT_PTR		nRet		= FALSE;

	if(!pSampleDlg)
	{
		pSampleDlg = (SampleDlg *)(LONG_PTR)GetWindowLongPtr(hDlg, DWLP_USER);
	}

	UNREFERENCED_PARAMETER(lParam);
	switch(message)
	{
		case WM_INITDIALOG:
			if(!pSampleDlg)
			{
				pSampleDlg = (SampleDlg *)lParam;
				SetWindowLongPtr(hDlg, DWLP_USER, (LONG)(LONG_PTR)pSampleDlg);
				nRet = pSampleDlg->OnInitDialog(hDlg);
			}
			break;

		case WM_COMMAND:
			if(LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
			{
				DestroyWindow(hDlg);
				nRet = TRUE;
				break;

			}

			switch(LOWORD(wParam))
			{
				case IDC_ACQUIRE_BTN:
					pSampleDlg->OnBtnAcquire();
					break;

				case IDC_SELSECT_SURCE_BIN:
					pSampleDlg->OnBtnSelectSource();
					break;

				case IDC_SHOWUI_CHECK:
					pSampleDlg->OnBtnUIModeChange();
					break;

				case IDC_SETTINGS_BTN:
					pSampleDlg->OnBtnSettings(hDlg);
					break;

				case IDC_CLOSE_BTN:
					DestroyWindow(hDlg);
					break;
			}
			break;

		case WM_XFERDATA:
			pSampleDlg->Update();
			break;

		case WM_ENDXFER:
			pSampleDlg->ScanFinish();
			break;

		case WM_PAINT:
			pSampleDlg->OnPaint();
			break;

		case WM_DESTROY:
			SetWindowLongPtr(hDlg, DWLP_USER, (LONG_PTR)NULL);
			PostQuitMessage(0);
			break;
	}

	return nRet;
}

void SampleDlg::OnBtnSelectSource()
{
	PostMessage(m_hDlg, (UINT)WM_SELECTSOURCE, NULL, NULL);
}

void SampleDlg::SetTransferManager(TransferManager *pTransferManager)
{
	m_pTransferManager = pTransferManager;
}

void SampleDlg::UpdateSourceTxt(LPCTSTR lpSourceName)
{
	SetDlgItemText(m_hDlg, IDC_SELSECT_SOURCE_STATIC, lpSourceName);

}